@@ -30,6 +30,10 @@ import pyopencl.cltypes  # noqa: F401
 import logging
 logger = logging.getLogger(__name__)
 
+# This supports ocl-icd find shipped OpenCL ICDs, cf.
+# https://github.com/isuruf/ocl-icd/commit/3862386b51930f95d9ad1089f7157a98165d5a6b
+# via
+# https://github.com/inducer/pyopencl/blob/0b3d0ef92497e6838eea300b974f385f94cb5100/scripts/build-wheels.sh#L43-L44
 import os
 os.environ["PYOPENCL_HOME"] = os.path.dirname(os.path.abspath(__file__))
 
@@ -257,6 +261,10 @@ class CompilerWarning(UserWarning):
     pass
 
 
+class CommandQueueUsedAfterExit(UserWarning):
+    pass
+
+
 def compiler_output(text):
     import os
     from warnings import warn
@@ -404,14 +412,6 @@ class Program:
                 self._prg = _cl._create_program_with_il(context, source)
                 return
 
-            import sys
-            if isinstance(source, str) and sys.version_info < (3,):
-                from warnings import warn
-                warn("Received OpenCL source code in Unicode, "
-                     "should be ASCII string. Attempting conversion.",
-                     stacklevel=2)
-                source = source.encode()
-
             self._context = context
             self._source = source
             self._prg = None
@@ -470,6 +470,9 @@ class Program:
                     logger.info("build program: kernel '%s' was part of a "
                             "lengthy %s (%.2f s)" % (attr, build_descr, duration))
 
+                # don't whine about build times more than once.
+                self._build_duration_info = None
+
             return knl
         except LogicError:
             raise AttributeError("'%s' was not found as a program "
@@ -710,6 +713,7 @@ def _add_functionality():
 
     def command_queue_exit(self, exc_type, exc_val, exc_tb):
         self.finish()
+        self._finalize()
 
     def command_queue_get_cl_version(self):
         return self.device._get_cl_version()
@@ -823,7 +827,7 @@ def _add_functionality():
                 self.function_name, self.num_args, self.num_args,
                 None,
                 warn_about_arg_count_bug=None,
-                work_around_arg_count_bug=None)
+                work_around_arg_count_bug=None, devs=self.context.devices)
 
         self._wg_info_cache = {}
         return self
@@ -862,7 +866,8 @@ def _add_functionality():
                         len(arg_types), self.num_args,
                         arg_types,
                         warn_about_arg_count_bug=warn_about_arg_count_bug,
-                        work_around_arg_count_bug=work_around_arg_count_bug)
+                        work_around_arg_count_bug=work_around_arg_count_bug,
+                        devs=self.context.devices)
 
         # Make ourselves a kernel-specific class, so that we're able to override
         # __call__. Inspired by https://stackoverflow.com/a/38541437
@@ -884,7 +889,7 @@ def _add_functionality():
         return result
 
     def kernel_set_args(self, *args, **kwargs):
-        # Need to dupicate the 'self' argument for dynamically generated  method
+        # Need to duplicate the 'self' argument for dynamically generated  method
         return self._set_args(self, *args, **kwargs)
 
     def kernel_call(self, queue, global_size, local_size, *args, **kwargs):
