@@ -29,19 +29,27 @@ FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 OTHER DEALINGS IN THE SOFTWARE.
 """
 
+from dataclasses import dataclass
+from typing import Optional
 
 import numpy as np
+
 import pyopencl as cl
-import pyopencl.array  # noqa
-from pyopencl.scan import ScanTemplate
+import pyopencl.array
+
+from pyopencl.elementwise import ElementwiseKernel
+from pyopencl.scan import ScanTemplate, GenericScanKernel
 from pyopencl.tools import dtype_to_ctype, get_arg_offset_adjuster_code
-from pytools import memoize, memoize_method, Record
+
+from pytools import memoize, memoize_method
 from mako.template import Template
 
 
 # {{{ "extra args" handling utility
 
 def _extract_extra_args_types_values(extra_args):
+    if extra_args is None:
+        extra_args = []
     from pyopencl.tools import VectorArg, ScalarArg
 
     extra_args_types = []
@@ -76,7 +84,7 @@ _copy_if_template = ScanTemplate(
         template_processor="printf")
 
 
-def copy_if(ary, predicate, extra_args=[], preamble="", queue=None, wait_for=None):
+def copy_if(ary, predicate, extra_args=None, preamble="", queue=None, wait_for=None):
     """Copy the elements of *ary* satisfying *predicate* to an output array.
 
     :arg predicate: a C expression evaluating to a `bool`, represented as a string.
@@ -124,7 +132,8 @@ def copy_if(ary, predicate, extra_args=[], preamble="", queue=None, wait_for=Non
 
 # {{{ remove_if
 
-def remove_if(ary, predicate, extra_args=[], preamble="", queue=None, wait_for=None):
+def remove_if(ary, predicate, extra_args=None, preamble="",
+        queue=None, wait_for=None):
     """Copy the elements of *ary* not satisfying *predicate* to an output array.
 
     :arg predicate: a C expression evaluating to a `bool`, represented as a string.
@@ -164,7 +173,8 @@ _partition_template = ScanTemplate(
         template_processor="printf")
 
 
-def partition(ary, predicate, extra_args=[], preamble="", queue=None, wait_for=None):
+def partition(ary, predicate, extra_args=None, preamble="",
+        queue=None, wait_for=None):
     """Copy the elements of *ary* into one of two arrays depending on whether
     they satisfy *predicate*.
 
@@ -229,7 +239,7 @@ _unique_template = ScanTemplate(
         template_processor="printf")
 
 
-def unique(ary, is_equal_expr="a == b", extra_args=[], preamble="",
+def unique(ary, is_equal_expr="a == b", extra_args=None, preamble="",
         queue=None, wait_for=None):
     """Copy the elements of *ary* into the output if *is_equal_expr*, applied to the
     array element and its predecessor, yields false.
@@ -415,10 +425,6 @@ RADIX_SORT_OUTPUT_STMT_TPL = Template(r"""//CL//
 
 # {{{ driver
 
-# import hoisted here to be used as a default argument in the constructor
-from pyopencl.scan import GenericScanKernel
-
-
 class RadixSort:
     """Provides a general `radix sort <https://en.wikipedia.org/wiki/Radix_sort>`_
     on the compute device.
@@ -429,7 +435,7 @@ class RadixSort:
     """
     def __init__(self, context, arguments, key_expr, sort_arg_names,
             bits_at_a_time=2, index_dtype=np.int32, key_dtype=np.uint32,
-            scan_kernel=GenericScanKernel, options=[]):
+            scan_kernel=GenericScanKernel, options=None):
         """
         :arg arguments: A string of comma-separated C argument declarations.
             If *arguments* is specified, then *input_expr* must also be
@@ -718,8 +724,14 @@ def _get_arg_list(arg_list, prefix=""):
     return result
 
 
-class BuiltList(Record):
-    pass
+@dataclass
+class BuiltList:
+    count: Optional[int]
+    starts: Optional[pyopencl.array.Array]
+    lists: Optional[pyopencl.array.Array]
+    num_nonempty_lists: Optional[int] = None
+    nonempty_indices: Optional[pyopencl.array.Array] = None
+    compressed_indices: Optional[pyopencl.array.Array] = None
 
 
 class ListOfListsBuilder:
@@ -762,9 +774,9 @@ class ListOfListsBuilder:
     """
     def __init__(self, context, list_names_and_dtypes, generate_template,
             arg_decls, count_sharing=None, devices=None,
-            name_prefix="plb_build_list", options=[], preamble="",
+            name_prefix="plb_build_list", options=None, preamble="",
             debug=False, complex_kernel=False,
-            eliminate_empty_output_lists=[]):
+            eliminate_empty_output_lists=False):
         """
         :arg context: A :class:`pyopencl.Context`.
         :arg list_names_and_dtypes: a list of `(name, dtype)` tuples
@@ -820,7 +832,6 @@ class ListOfListsBuilder:
             Change *eliminate_empty_output_lists* argument type from `bool` to
             `list`.
         """
-
         if devices is None:
             devices = context.devices
 
@@ -874,7 +885,6 @@ class ListOfListsBuilder:
 
     @memoize_method
     def get_scan_kernel(self, index_dtype):
-        from pyopencl.scan import GenericScanKernel
         return GenericScanKernel(
                 self.context, index_dtype,
                 arguments="__global %s *ary" % dtype_to_ctype(index_dtype),
@@ -894,7 +904,6 @@ class ListOfListsBuilder:
         """
         arguments = Template(arguments)
 
-        from pyopencl.scan import GenericScanKernel
         return GenericScanKernel(
                 self.context, index_dtype,
                 arguments=arguments.render(index_t=dtype_to_ctype(index_dtype)),
@@ -925,7 +934,7 @@ class ListOfListsBuilder:
                 if name not in self.count_sharing]
 
         user_list_args = []
-        for name, dtype in self.list_names_and_dtypes:
+        for name, _dtype in self.list_names_and_dtypes:
             if name in self.count_sharing:
                 continue
 
@@ -1157,7 +1166,7 @@ class ListOfListsBuilder:
 
         # {{{ allocate memory for counts
 
-        for name, dtype in self.list_names_and_dtypes:
+        for name, _dtype in self.list_names_and_dtypes:
             if name in self.count_sharing:
                 continue
             if name in omit_lists:
@@ -1196,7 +1205,7 @@ class ListOfListsBuilder:
                 wait_for=wait_for)
 
         compress_events = {}
-        for name, dtype in self.list_names_and_dtypes:
+        for name, _dtype in self.list_names_and_dtypes:
             if name in omit_lists:
                 continue
             if name in self.count_sharing:
@@ -1228,7 +1237,7 @@ class ListOfListsBuilder:
 
         scan_events = []
 
-        for name, dtype in self.list_names_and_dtypes:
+        for name, _dtype in self.list_names_and_dtypes:
             if name in self.count_sharing:
                 continue
             if name in omit_lists:
@@ -1310,8 +1319,11 @@ class ListOfListsBuilder:
 
 # {{{ key-value sorting
 
-class _KernelInfo(Record):
-    pass
+@dataclass(frozen=True)
+class _KernelInfo:
+    by_target_sorter: RadixSort
+    start_finder: ElementwiseKernel
+    bound_propagation_scan: GenericScanKernel
 
 
 def _make_cl_int_literal(value, dtype):
@@ -1353,7 +1365,6 @@ class KeyValueSorter:
 
     @memoize_method
     def get_kernels(self, key_dtype, value_dtype, starts_dtype):
-        from pyopencl.algorithm import RadixSort
         from pyopencl.tools import VectorArg, ScalarArg
 
         by_target_sorter = RadixSort(
@@ -1384,7 +1395,6 @@ class KeyValueSorter:
                             ),
                         var_values=())
 
-        from pyopencl.scan import GenericScanKernel
         bound_propagation_scan = GenericScanKernel(
                 self.context, starts_dtype,
                 arguments=[
