@@ -637,8 +637,8 @@ def test_vector_args(ctx_factory):
 def test_header_dep_handling(ctx_factory):
     context = ctx_factory()
 
-    from os.path import exists
-    assert exists("empty-header.h")  # if this fails, change dir to pyopencl/test
+    from os.path import exists, dirname, join
+    assert exists(join(dirname(__file__), "empty-header.h"))
 
     kernel_src = """
     #include <empty-header.h>
@@ -648,10 +648,8 @@ def test_header_dep_handling(ctx_factory):
     }
     """
 
-    import os
-
-    cl.Program(context, kernel_src).build(["-I", os.getcwd()])
-    cl.Program(context, kernel_src).build(["-I", os.getcwd()])
+    cl.Program(context, kernel_src).build(["-I", dirname(__file__)])
+    cl.Program(context, kernel_src).build(["-I", dirname(__file__)])
 
 
 def test_context_dep_memoize(ctx_factory):
@@ -1247,6 +1245,15 @@ def test_empty_ndrange(ctx_factory, empty_shape):
     prg.add_two(queue, a.shape, None, a.data, allow_empty_ndrange=True)
 
 
+def test_command_queue_context_manager(ctx_factory):
+    ctx = ctx_factory()
+    with cl.CommandQueue(ctx) as queue:
+        q = queue
+
+    with pytest.warns(cl.CommandQueueUsedAfterExit):
+        q.flush()
+
+
 if __name__ == "__main__":
     # make sure that import failures get reported, instead of skipping the tests.
     import pyopencl  # noqa
